library(tidyr)
distribution <- function(
  DATA_IN,
  INSTRUMENT,
  SIZE_CLASSES
){
  out <- dplyr::filter(DATA_IN, Instrument == INSTRUMENT) %>% 
    dplyr::select(!Instrument) %>% 
    t() %>% janitor::row_to_names(row_number = 1)
  out <- data.frame(out) %>% sapply(as.numeric)
  # out <- out/out[,ncol(out)] # normalize to channel size
  out <- data.frame(out, SIZE_CLASSES)
}



library(scales)
library(ggplot2)
library(latex2exp)
library(forcats)
# https://colorbrewer2.org/#type=diverging&scheme=RdBu&n=6
printer_friendly_palette <- c("#b2182b", "#ef8a62", "#fddbc7", "#d1e5f0", "#67a9cf", "#2166ac")
printer_friendly_palette_box <- c("#ca0020", "#f4a582", "#92c5de", "#0571b0", "#ffffff", '#808080')

# dplyr::filter(winds.data.aggregated.medians.wisi,Condition=='breathing' |
#                                                              Condition=='speaking'
#                )

plot_x = 2.54*6#6.375 # plot size in inch
plot_y = plot_x*2/3. # plot size in inch
ft.size = 16 # font size in plots in pt
plotID <- function(
  DATA_IN,
  X_VALUES,
  Y_VALUES,
  GROUPS,
  Y_RANGE,
  Y_LABEL
){
  p0 <- ggplot(data = DATA_IN, aes(x=X_VALUES, y=Y_VALUES, fill=GROUPS))
  p0 <- p0 + geom_boxplot(outlier.shape=8,
                          outlier.size=.5,
                          notch=FALSE,
                          lwd=.2,
                          fatten = 4)

  p0 <- p0 + scale_y_continuous(trans = log10_trans(),
                                labels = trans_format("log10",
                                                      math_format(10^.x)),
                                limits = Y_RANGE,
                                expand = expansion(mult = c(0, 0)))
  p0 <- p0 + xlab(TeX(""))
  p0 <- p0 + ylab(TeX(Y_LABEL))
  p0 <- p0 + theme_bw()
  p0 <- p0 + theme(panel.grid.minor = element_line(colour = "#FFFFFF"),
                   panel.grid.major = element_line(colour = "#FFFFFF"))
  p0 <- p0 + theme(legend.title = element_blank())
  p0 <- p0 + theme(legend.position = c(0.5, 0.09),
                   plot.margin = margin(9, 1, 1, 1, "pt"),
                   panel.border = element_rect(fill=NA, colour = "black",
                                               size=1),
                   text = element_text(size=ft.size),
                   legend.text=element_text(size=ft.size),
                   legend.direction = "horizontal",
                   axis.text.x = element_text(colour = "black",size=ft.size,angle = 0),
                   axis.text.y = element_text(colour = "black",size=ft.size),
                   legend.spacing.x = unit(.05, 'cm'),
                   legend.key.size = unit(.7,"line"),
                   legend.key = element_rect(colour = "transparent", fill = "white"),
                   legend.background=element_blank()
  )
  p0 <- p0 + annotation_logticks(sides="lr",
                                 short = unit(0.1, "cm"),
                                 mid = unit(0.15, "cm"),
                                 long = unit(0.2, "cm"),
                                 size = 0.3)
  p0 <- p0 + scale_fill_manual(values=printer_friendly_palette_box)
  p0 <- p0 + scale_color_manual(values=printer_friendly_palette_box)
  p0
}

plotID_lin <- function(
  DATA_IN,
  X_VALUES,
  Y_VALUES,
  GROUPS,
  Y_RANGE,
  Y_LABEL
){
  
  p0 <- ggplot(data = DATA_IN, aes(x=X_VALUES, y=Y_VALUES, fill=GROUPS))
  p0 <- p0 + geom_boxplot(outlier.shape=8,
                          outlier.size=.5,
                          notch=FALSE,
                          lwd=.2,
                          fatten = 4)
  p0 <- p0 + xlab(TeX(""))
  p0 <- p0 + ylab(TeX(Y_LABEL)) + ylim(Y_RANGE)
  p0 <- p0 + theme_bw()
  p0 <- p0 + theme(panel.grid.minor = element_line(colour = "#FFFFFF"),
                   panel.grid.major = element_line(colour = "#FFFFFF"))
  p0 <- p0 + theme(legend.title = element_blank())
  p0 <- p0 + theme(legend.position = c(0.5, 0.09),
                   plot.margin = margin(6, 6, 1, 1, "pt"),
                   panel.border = element_rect(fill=NA, colour = "black",
                                               size=1),
                   text = element_text(size=ft.size),
                   legend.text=element_text(size=ft.size),
                   legend.direction = "horizontal",
                   axis.text.x = element_text(colour = "black",size=ft.size,angle = 0),
                   axis.text.y = element_text(colour = "black",size=ft.size),
                   legend.spacing.x = unit(.05, 'cm'),
                   legend.key.size = unit(.7,"line"),
                   legend.key = element_rect(colour = "transparent", fill = "white"),
                   legend.background=element_blank()
  )
  p0 <- p0 + scale_fill_manual(values=printer_friendly_palette_box)
  p0 <- p0 + scale_color_manual(values=printer_friendly_palette_box)
  p0
}

plotID_dist_bar <- function(
  DATA_IN,
  TITLE,
  Y_LABEL,
  X_LABEL,
  Y_RANGE
){
  DATA_MEAN <- DATA_IN$mean
  DATA_EB <- DATA_IN$eb
  
  DATA_EB[DATA_MEAN<1] <- NA # ignore non-physical mean values lower than 1
  DATA_MEAN[DATA_MEAN<1] <- NA # ignore non-physical values lower than 1
  
  X_VALUES <- DATA_IN$mean$size_categories
  data_mean <- data.frame(x = X_VALUES,                            # Reshape data frame
                         y = c(DATA_MEAN$breathing, 
                               DATA_MEAN$speaking, 
                               DATA_MEAN$piano,
                               DATA_MEAN$forte,
                               DATA_MEAN$Ode,
                               DATA_MEAN$orchestral_excerpt),
                         group = c(rep("breathing", nrow(DATA_MEAN)),
                                   rep("speaking", nrow(DATA_MEAN)),
                                   rep("piano", nrow(DATA_MEAN)),
                                   rep("forte", nrow(DATA_MEAN)),
                                   rep("Ode", nrow(DATA_MEAN)),
                                   rep("orchestral_excerpt", nrow(DATA_MEAN))
                         )
                         
  )
  data_eb <- data.frame(eb = c(DATA_EB$breathing, 
                               DATA_EB$speaking, 
                               DATA_EB$piano,
                               DATA_EB$forte,
                               DATA_EB$Ode,
                               DATA_EB$orchestral_excerpt)
  )  
  
  data_ggp <- cbind(data_mean,data_eb)

  data_ggp$group <- factor(data_ggp$group,
                           levels = c('forte',
                                      'Ode',
                                      'orchestral_excerpt',
                                      'piano','speaking','breathing'))


  p0 <- ggplot(data_ggp, aes(x, y, fill = group)) 
  p0 <- p0 + geom_col(position=position_dodge(width = 0.75),width=0.65,lwd=.1)
  p0 <- p0 + geom_errorbar(aes(x, ymin=y-eb,ymax=y+eb),width=.4,color="black",size=1.3,
                           position=position_dodge(width = 0.75))
  p0 <- p0 + scale_x_discrete(labels=c("C1" = ">0.3-0.5",
                                       "C2" = ">0.5-1.0",
                                       "C3" = ">1.0-3.0",
                                       "C4" = ">3.0-5.0",
                                       "C5" = ">5.0-10.0",
                                       "C6" = ">10.0-25.0"
  ))



  p0 <- p0 +labs(x =TeX(X_LABEL), y = TeX(Y_LABEL))
  p0 <- p0 + theme_bw()+ ylim(Y_RANGE)
  p0 <- p0 + theme(panel.grid.minor = element_line(colour = "#FFFFFF"),
                   panel.grid.major = element_line(colour = "#FFFFFF"))
  p0 <- p0 + theme(legend.position = c(0.8, 0.75),
                   plot.margin = margin(6, 1, 1, 1, "pt"),
                   panel.border = element_rect(fill=NA, colour = "black",
                                               linewidth=1),
                   text = element_text(size=ft.size),
                   legend.text=element_text(size=ft.size),
                   legend.direction = "vertical",
                   axis.text.x = element_text(colour = "black",size=ft.size,angle = 0),
                   axis.text.y = element_text(colour = "black",size=ft.size),
                   legend.spacing.x = unit(.05, 'cm'),
                   legend.key.size = unit(.7,"line"),
                   legend.key = element_rect(colour = "transparent", fill = "white"),
                   legend.background=element_blank()
  )

  p0 <- p0 + scale_y_log10(labels = trans_format("log10",
                                                 math_format(10^.x)),
                           limits = Y_RANGE,
                           )

  p0 <- p0 + guides(fill=guide_legend(title=TITLE))
  p0 <- p0 + scale_fill_manual(values=printer_friendly_palette)
  p0 <- p0 + scale_color_manual(values=printer_friendly_palette)
  p0
}

library(xtable)
library(rstatix)
library(tidyr)
create_table_instrument <- function(
  DATA_IN,
  MEASURE,
  CONDITION
){
  table <- dplyr::filter(DATA_IN, Condition == CONDITION) %>% 
    group_by(Instrument) %>% 
    rstatix::get_summary_stats(all_of(MEASURE), 
                               type = "common", 
                               show = c('min','median','max'))
  table <- data.frame(table$Instrument,
                      table$min,
                      table$median,
                      table$max
  )
  names(table) <- c('ID','Min','Median','Max')
  table <- table
}

calc_mean_eb <- function(
    DATA_IN,
    INSTRUMENT
){
  mean <- aggregate(. ~ Instrument+Condition,DATA_IN$data, mean)
  eb <- aggregate(. ~ Instrument+Condition, DATA_IN$data, 
                                FUN = function(x) c(se=plotrix::std.error(x)))
  list('mean'=distribution(
    mean,
    INSTRUMENT,
    DATA_IN$size_categories),
    'eb'=distribution(
      eb,
      INSTRUMENT,
      DATA_IN$size_categories)
  )
}
